#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Python -*-

"""
 @file RoboQueJudge.py
 @brief Estimate distance of robots and Judge win/loose of the game ROBO-QUE 
 @date $Date$


"""
import sys
import time
sys.path.append(".")

# Import RTM module
import RTC
import OpenRTM_aist

import numpy as np

# Import Service implementation class
# <rtc-template block="service_impl">

# </rtc-template>

# Import Service stub modules
# <rtc-template block="consumer_import">
# </rtc-template>


# This module's spesification
# <rtc-template block="module_spec">
roboquejudge_spec = ["implementation_id", "RoboQueJudge", 
		 "type_name",         "RoboQueJudge", 
		 "description",       "Estimate distance of robots and Judge win/loose of the game ROBO-QUE ", 
		 "version",           "1.0.1", 
		 "vendor",            "ota", 
		 "category",          "Applicatio", 
		 "activity_type",     "STATIC", 
		 "max_instance",      "1", 
		 "language",          "Python", 
		 "lang_type",         "SCRIPT",
		 "conf.default.map_adjust_x", "1000",
		 "conf.default.map_adjust_y", "1000",

		 "conf.__widget__.map_adjust_x", "slider",
		 "conf.__widget__.map_adjust_y", "slider",

         "conf.__type__.map_adjust_x", "int",
         "conf.__type__.map_adjust_y", "int",

		 ""]
# </rtc-template>

##
# @class RoboQueJudge
# @brief Estimate distance of robots and Judge win/loose of the game ROBO-QUE 
# 
# 
class RoboQueJudge(OpenRTM_aist.DataFlowComponentBase):
	
	##
	# @brief constructor
	# @param manager Maneger Object
	# 
	def __init__(self, manager):
		OpenRTM_aist.DataFlowComponentBase.__init__(self, manager)

		pos1_arg = [None] * ((len(RTC._d_TimedPoint2D) - 4) / 2)
		self._d_pos1 = RTC.TimedPoint2D(RTC.Time(0,0), RTC.Point2D(0.0,0.0))
		"""
		"""
		self._pos1In = OpenRTM_aist.InPort("pos1", self._d_pos1)
		pos2_arg = [None] * ((len(RTC._d_TimedPoint2D) - 4) / 2)
		self._d_pos2 = RTC.TimedPoint2D(RTC.Time(0,0), RTC.Point2D(0.0,0.0))
		"""
		"""
		self._pos2In = OpenRTM_aist.InPort("pos2", self._d_pos2)
		image_arg = [None] * ((len(RTC._d_CameraImage) - 4) / 2)
		self._d_image = RTC.CameraImage(*image_arg)
		"""
		"""
		self._imageIn = OpenRTM_aist.InPort("image", self._d_image)
		distance_arg = [None] * ((len(RTC._d_TimedDoubleSeq) - 4) / 2)
		self._d_distance = RTC.TimedDoubleSeq(RTC.Time(0,0), [])
		"""
		"""
		self._distanceOut = OpenRTM_aist.OutPort("distance", self._d_distance)
		outcome_arg = [None] * ((len(RTC._d_TimedBooleanSeq) - 4) / 2)
		self._d_outcome = RTC.TimedBooleanSeq(RTC.Time(0,0), [])
		"""
		"""
		self._outcomeOut = OpenRTM_aist.OutPort("outcome", self._d_outcome)

		self.log = OpenRTM_aist.Manager.instance().getLogbuf("RoboQueJudgeRTC")
		
		self.max_distance = 1000.0
		self.glid_x=5 #-1 # *2-1
		self.glid_y=4 # *2-1
#map=[[0b0000 for i in range(glid_x)] for j in range(glid_y)]
		self.map=[[0b0000, 0b0100, 0b0010, 0b1000],
				  [0b0100, 0b0011, 0b1100, 0b0000],
				  [0b0011, 0b1010, 0b1011, 0b1010],
				  [0b0000, 0b0110, 0b1000, 0b0100],
				  [0b0000, 0b0001, 0b0000, 0b0001]]
		#self.thresh=1
		self.img_width=320
		self.img_height=240

		self.x1=0.0
		self.y1=0.0
		self.x2=1000.0
		self.y2=800.0

		# initialize of configuration-data.
		# <rtc-template block="init_conf_param">
		"""
		
		 - Name:  map_adjust_x
		 - DefaultValue: 1000
		"""
		self._map_adjust_x = [1000]
		"""
		
		 - Name:  map_adjust_y
		 - DefaultValue: 1000
		"""
		self._map_adjust_y = [700]
		
		# </rtc-template>

	def calcDistance(self,x1,y1,x2,y2):
		dist = np.sqrt((x1-x2)**2 + (y1-y2)**2)
		return dist
	
	def adjustRange(self,x,range_x):
		if x<0:
			x=0
		elif x > range_x:
			x=range_x
		#print"adjusted ", x
		return x
	
	def judge(self,map,chaser_x,chaser_y,runaway_x,runaway_y,img_width,img_height,glid_x,glid_y):
# 	    chaser_x=chaser_x*1.0/img_width*glid_x
# 	    chaser_y=chaser_y*1.0/img_height*glid_y
# 	    runaway_x=runaway_x*1.0/img_width*glid_x
# 	    runaway_y=runaway_y*1.0/img_height*glid_y
	    chaser_x=int(chaser_x*glid_x)
	    chaser_y=int(chaser_y*glid_y)
	    runaway_x=int(runaway_x*glid_x)
	    runaway_y=int(runaway_y*glid_y)
	    
	    print("    oni:  "+str(chaser_x)+","+str( chaser_y))
	    print("   nige:  "+str(runaway_x)+","+str(runaway_y))
	    
	    diff_x=int(chaser_x-runaway_x)
	    diff_y=int(chaser_y-runaway_y)
	    print("   diff:  "+str(diff_x)+","+str(diff_y))
	    
	    chaser_x=self.adjustRange(chaser_x,glid_x-1)
	    chaser_y=self.adjustRange(chaser_y,glid_y-1)
	    runaway_x=self.adjustRange(runaway_x,glid_x-1)
	    runaway_y=self.adjustRange(runaway_y,glid_y-1)
	    #import pdb; pdb.set_trace()
	    print "MAP        {0:b} {1:b}".format(
			map[chaser_x][chaser_y],
			map[runaway_x][runaway_y])
	    if diff_x==0:
	    	print("diff_x = 0")
	        if diff_y==0:
	            print("Caught!!")
	            return True
        	print("diff_y = {0}; map = {1:b}".format(
				diff_y,
				map[chaser_x][chaser_y]))
        	print("0b0010 test: &0b0010 = {0}, res = {1}".format(
				bin(map[chaser_x][chaser_y] & 0b1000),
				map[chaser_x][chaser_y] & 0b0010 == 0))
        	print("0b1000 test: &0b1000 = {0}, res = {1}".format(
				bin(map[chaser_x][chaser_y] & 0b0010),
				map[chaser_x][chaser_y] & 0b1000 == 0))
	        if diff_y==1 and map[chaser_x][chaser_y] & 0b1000 == 0:
	        	print("Caught!!")
	        	return True
	        if diff_y==-1 and map[chaser_x][chaser_y] & 0b0010 == 0:
	        	print("Caught!!")
	        	return True
	    if diff_y==0:
        	print("diff_x = {0}; map = {1:b}".format(
				diff_x,
				map[chaser_x][chaser_y]))
        	print("0b0001 test: &0b0001 = {0}, res = {1}".format(
				bin(map[chaser_x][chaser_y] & 0b0001),
				map[chaser_x][chaser_y] & 0b0001 == 0))
        	print("0b0100 test: &0b0100 = {0}, res = {1}".format(
				bin(map[chaser_x][chaser_y] & 0b0100),
				map[chaser_x][chaser_y] & 0b0100 == 0))
	        if diff_x==1 and map[chaser_x][chaser_y] & 0b0001 == 0:
	        	print("Caught!!")
	        	return True
	        if diff_x==-1 and map[chaser_x][chaser_y] & 0b0100 == 0:
	        	print("Caught!!")
	        	return True
	    print("Run!")
	    return False
	##
	#
	# The initialize action (on CREATED->ALIVE transition)
	# formaer rtc_init_entry() 
	# 
	# @return RTC::ReturnCode_t
	# 
	#
	def onInitialize(self):
		# Bind variables and configuration variable
		
		# Set InPort buffers
		self.addInPort("pos1",self._pos1In)
		self.addInPort("pos2",self._pos2In)
		self.addInPort("image",self._imageIn)
		
		# Set OutPort buffers
		self.addOutPort("distance",self._distanceOut)
		self.addOutPort("outcome",self._outcomeOut)
		
		# Set service provider to Ports
		
		# Set service consumers to Ports
		
		# Set CORBA Service Ports
		
		return RTC.RTC_OK
	
	#	##
	#	# 
	#	# The finalize action (on ALIVE->END transition)
	#	# formaer rtc_exiting_entry()
	#	# 
	#	# @return RTC::ReturnCode_t
	#
	#	# 
	#def onFinalize(self):
	#
	#	return RTC.RTC_OK
	
	#	##
	#	#
	#	# The startup action when ExecutionContext startup
	#	# former rtc_starting_entry()
	#	# 
	#	# @param ec_id target ExecutionContext Id
	#	#
	#	# @return RTC::ReturnCode_t
	#	#
	#	#
	#def onStartup(self, ec_id):
	#
	#	return RTC.RTC_OK
	
	#	##
	#	#
	#	# The shutdown action when ExecutionContext stop
	#	# former rtc_stopping_entry()
	#	#
	#	# @param ec_id target ExecutionContext Id
	#	#
	#	# @return RTC::ReturnCode_t
	#	#
	#	#
	#def onShutdown(self, ec_id):
	#
	#	return RTC.RTC_OK
	
		##
		#
		# The activated action (Active state entry action)
		# former rtc_active_entry()
		#
		# @param ec_id target ExecutionContext Id
		# 
		# @return RTC::ReturnCode_t
		#
		#
	def onActivated(self, ec_id):
		#self.log.RTC_INFO("log test")
		print"[JudgeRTC] Activated"
		self._d_distance.data.append(self.max_distance)
		self._d_outcome.data.append(0)
		
		return RTC.RTC_OK
	
		##
		#
		# The deactivated action (Active state exit action)
		# former rtc_active_exit()
		#
		# @param ec_id target ExecutionContext Id
		#
		# @return RTC::ReturnCode_t
		#
		#
	def onDeactivated(self, ec_id):
	
		return RTC.RTC_OK
	
		##
		#
		# The execution action that is invoked periodically
		# former rtc_active_do()
		#
		# @param ec_id target ExecutionContext Id
		#
		# @return RTC::ReturnCode_t
		#
		#
	def onExecute(self, ec_id):
		#import pdb; pdb.set_trace()
		#self._d_distance.data.append(self.max_distance)
		
		if self._pos1In.isNew():
			self._d_pos1=self._pos1In.read()
			print "pos1: ", self._d_pos1.data
			self.x1=self._d_pos1.data.x*1000/self._map_adjust_x[0]
			self.y1=self._d_pos1.data.y*1000/self._map_adjust_y[0]
			
			if self._pos2In.isNew():
				self._d_pos2=self._pos2In.read()
				print "pos2: ", self._d_pos2.data
				self.x2=self._d_pos2.data.x*1000/self._map_adjust_x[0]
				self.y2=self._d_pos2.data.y*1000/self._map_adjust_y[0]

# 				self._d_distance.data.append(self.calcDistance(self._d_pos1.data.x,self._d_pos1.data.y,self._d_pos2.data.x,self._d_pos2.data.y))
# 				print "    pos1:", self._d_pos1.data
# 				print "    pos2:", self._d_pos2.data
#  
# 				self._d_outcome.data.append(self.judge(self.map,
# 													   self._d_pos1.data.x,
# 													   self._d_pos1.data.y,
# 													   self._d_pos2.data.x,
# 													   self._d_pos2.data.y,
# 													   self.img_width,
# 													   self.img_height,
# 													   self.glid_x,
# 													   self.glid_y))
		self._d_distance.data[0]=self.calcDistance(self.x1,self.y1,self.x2,self.y2)
		self._d_outcome.data[0]=(self.judge(self.map,
											   self.x1,
											   self.y1,
											   self.x2,
											   self.y2,
											   self.img_width,
											   self.img_height,
											   self.glid_x,
											   self.glid_y))
		#
		if self._d_distance.data[0]<0.22:
			self._d_outcome.data[0]=True
			print("Caught! (distance)")
			
		#
		print "adjusted pos1: ", self.x1, self.y1, "        pos2: ", self.x2, self.y2
		#print "map pos1: ", self.x1/self._map_adjust_x, self.y1/self._map_adjust_y,  "        pos2: ", self.x2/self._map_adjust_x, self.y2/self._map_adjust_y
		print "distance: "+str(self._d_distance.data[0])

		print "outcome : " +str(self._d_outcome.data[0])
		print "------------------------------"
		self._distanceOut.write()
		self._outcomeOut.write()
		
		#self.log.RTC_INFO("log test")
		#if(err):
		#	self.log.RTC_ERR("Not registered"+str(errno))
			
		return RTC.RTC_OK
	
	#	##
	#	#
	#	# The aborting action when main logic error occurred.
	#	# former rtc_aborting_entry()
	#	#
	#	# @param ec_id target ExecutionContext Id
	#	#
	#	# @return RTC::ReturnCode_t
	#	#
	#	#
	#def onAborting(self, ec_id):
	#
	#	return RTC.RTC_OK
	
	#	##
	#	#
	#	# The error action in ERROR state
	#	# former rtc_error_do()
	#	#
	#	# @param ec_id target ExecutionContext Id
	#	#
	#	# @return RTC::ReturnCode_t
	#	#
	#	#
	#def onError(self, ec_id):
	#
	#	return RTC.RTC_OK
	
	#	##
	#	#
	#	# The reset action that is invoked resetting
	#	# This is same but different the former rtc_init_entry()
	#	#
	#	# @param ec_id target ExecutionContext Id
	#	#
	#	# @return RTC::ReturnCode_t
	#	#
	#	#
	#def onReset(self, ec_id):
	#
	#	return RTC.RTC_OK
	
	#	##
	#	#
	#	# The state update action that is invoked after onExecute() action
	#	# no corresponding operation exists in OpenRTm-aist-0.2.0
	#	#
	#	# @param ec_id target ExecutionContext Id
	#	#
	#	# @return RTC::ReturnCode_t
	#	#

	#	#
	#def onStateUpdate(self, ec_id):
	#
	#	return RTC.RTC_OK
	
	#	##
	#	#
	#	# The action that is invoked when execution context's rate is changed
	#	# no corresponding operation exists in OpenRTm-aist-0.2.0
	#	#
	#	# @param ec_id target ExecutionContext Id
	#	#
	#	# @return RTC::ReturnCode_t
	#	#
	#	#
	#def onRateChanged(self, ec_id):
	#
	#	return RTC.RTC_OK
	



def RoboQueJudgeInit(manager):
    profile = OpenRTM_aist.Properties(defaults_str=roboquejudge_spec)
    manager.registerFactory(profile,
                            RoboQueJudge,
                            OpenRTM_aist.Delete)

def MyModuleInit(manager):
    RoboQueJudgeInit(manager)

    # Create a component
    comp = manager.createComponent("RoboQueJudge")

def main():
	mgr = OpenRTM_aist.Manager.init(sys.argv)
	mgr.setModuleInitProc(MyModuleInit)
	mgr.activateManager()
	mgr.runManager()

if __name__ == "__main__":
	main()

